/**
* Filename: syncmab.js
* Created:  10.04.2006
* Creator:  Jan Pavek <jan@pavek.de>
*
* Revisions:
*   02.03.2007 - jp
*
*/

var maxMabCount = 6;

var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
var directory = rdf.GetResource("moz-abdirectory://").QueryInterface(Components.interfaces.nsIAbDirectory);
var directoryService =  Components.classes['@mozilla.org/file/directory_service;1'].
    getService(Components.interfaces.nsIProperties);
var profileFolder = directoryService.get('ProfD',  Components.interfaces.nsIFile);
var profileDir = profileFolder.QueryInterface(Components.interfaces.nsILocalFile);

var mabSyncHandler = new Array( maxMabCount );


function syncAddressBooks( runType )
{
    var syncType;
    var filePath;
    var mabCount = 0;
    var syncStatus = 2;
    var useAutoSync = false;
    var useSyncTime = false;
    var syncTimeStamp = 0;
    var syncTime = 0;

    try{ useAutoSync = pref.getBoolPref( "SyncMab.useAutoSync" ); } catch(e){}
    try{ useSyncTime = pref.getBoolPref( "SyncMab.useSyncTime" ); } catch(e){}
    try{ syncTimeStamp = pref.getIntPref( "SyncMab.syncTimeStamp" ); } catch(e){}
    try{ syncTime = pref.getIntPref( "SyncMab.syncTime" ); } catch(e){}

    if( useAutoSync )
    {
        syncStatus = 1;

        if( syncTime <= 0 )
            syncTime = 1;

        syncTime = syncTime * 86400;

        if( useSyncTime &&
            (syncTimeStamp + syncTime) > getTimeStamp() )
            syncStatus = 2;
    } // if

    if( runType == undefined || syncStatus != 2 )
    {
        useWaitWindow = false;

        for( mabCount = 0; mabCount < maxMabCount; mabCount++ )
        {
            mabSync = new MabSyncData();
            mabSync.load( mabCount );

            if( mabSync.syncActive && mabSync.filePath != "" )
            {
                if( runType == undefined ||
                    (runType == "onunload" && mabSync.syncType == "export" ) )
                {
                    if( mabSync.syncLocation != "file" )
                    {
                        useWaitWindow = true;
                        break;
                    } // if
                } // if
            } // if

        } // for

        if( useWaitWindow )
        {
            waitWindow = window.open("chrome://syncmab/content/syncwait.xul", "_blank", "chrome,titlebar");
            waitWindow.waitRunType = runType;
        }
        else
            syncStatus = syncAddressBooksNow( runType );

    } // if

    return syncStatus;

} // syncAddressBooks



function syncAddressBooksNow( runType )
{
    syncStatus = 1;

    for( mabCount = 0; mabCount < maxMabCount; mabCount++ )
    {
        mabSync = new MabSyncData();
        mabSync.load( mabCount );

        if( mabSync.syncActive && mabSync.filePath != "" )
        {
            if( runType == undefined ||
                (runType == "onload" && mabSync.syncType == "import" ) ||
                (runType == "onunload" && mabSync.syncType == "export" ) )
            {
                if( syncOneAddressBook( mabSync ) == false )
                    syncStatus = 0;
            } // if
        } // if

    } // for

    return syncStatus;

} // syncAddressBooksNow



function syncOneAddressBook( mabSync )
{
    var syncDone = true;

    switch( mabSync.syncLocation )
    {
    case 'file':
        syncDone = syncOneAddressBookViaFile( mabSync );
        break;
    case 'ftp':
    case 'http':
    case 'https':
        syncDone = syncOneAddressBookViaFtp( mabSync );
        break;
    } // switch

    if( syncDone )
        pref.setIntPref( "SyncMab.syncTimeStamp", getTimeStamp() );

    return syncDone;

} // syncOneAddressBook



function syncOneAddressBookViaFile( mabSync )
{
    var syncDone = true;

    var externMabFile = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);

    try
    {
      externMabFile.initWithPath( mabSync.filePath );

      if( mabSync.syncType == "export" )
      {
        if( mabSync.mabToExport == '' )
          syncDone = exportAllAddressBooksViaFile( externMabFile, mabSync.importNew );
        else
          syncDone = exportAddressBookViaFile( mabSync.mabToExport, externMabFile, mabSync.importNew );
      }
      else
      if( mabSync.syncType == "import" )
      {
        if( externMabFile.exists() &&
            externMabFile.isDirectory() )
          syncDone = importAllAddressBooksViaFile( externMabFile, mabSync.exportNew );
        else
          syncDone = importAddressBookViaFile( externMabFile, mabSync.mabToImport, mabSync.exportNew );
      } // if
    }
    catch(e)
    {
      syncDone = false;
    } // catch
    
    return syncDone;

} // syncOneAddressBookViaFile



function syncOneAddressBookViaFtp( mabSync )
{
    var ret = false;

    if( mabSyncHandler[mabSync.syncId] == undefined ||
        mabSyncHandler[mabSync.syncId] == null )
    {
        mabSyncHandler[mabSync.syncId] = new SyncMabHandlerFTP( mabSync );

        if( mabSync.syncType == "export" )
            ret = mabSyncHandler[mabSync.syncId].push();
        else
            ret = mabSyncHandler[mabSync.syncId].pull();

        if( ret == false )
            mabSyncHandler[mabSync.syncId] = null;
    } // if

    return ret;

} // syncOneAddressBookViaFtp



function createAddressBook( name, fileName )
{
    var abProps = Components.classes["@mozilla.org/addressbook/properties;1"].
        createInstance(Components.interfaces.nsIAbDirectoryProperties);
    abProps.description = name;
    abProps.dirType = 2;
    abProps.fileName = fileName;
    directory.createNewDirectory( abProps );

} // createAddressBook



function getTimeStamp()
{
    var date = new Date();
    return Math.floor( date.getTime() / 1000 );

} // getTimeStamp



function deleteAddressBook( fileName, removeFile )
{
    var cn = directory.childNodes;
    var aBook = cn.getNext();

    while( aBook != null )
    {
        var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);
        var mabFileName = cur.directoryProperties.fileName;
        if( mabFileName == fileName )
        {
            var prefName = cur.directoryProperties.prefName;
            directory.deleteDirectory( cur );
            pref.deleteBranch( prefName );
            
            if( removeFile == true )
            {
              var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
              localMabFile.appendRelativePath( mabFileName );
              if( localMabFile.exists() )
                localMabFile.remove(false);
            } // if
            
            break;
        } // if

        try
        {
            aBook = cn.getNext();
        }
        catch(e)
        {
            break;
        } // catch
    } // while

} // deleteAddressBook



function getAddressBook( fileName )
{
    var cn = directory.childNodes;
    var aBook = cn.getNext();

    while( aBook != null )
    {
        var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);
        if( cur.directoryProperties.fileName == fileName )
            return cur;

        try
        {
            aBook = cn.getNext();
        }
        catch(e)
        {
            break;
        } // catch
    } // while

} // getAddressBook



function getAddressBookByName( name )
{
    var cn = directory.childNodes;
    var aBook = cn.getNext();

    while( aBook != null )
    {
        var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);
        if( cur.directoryProperties.description == name )
            return cur;

        try
        {
            aBook = cn.getNext();
        }
        catch(e)
        {
            break;
        } // catch
    } // while

} // getAddressBookByName



function getAddressBookNameFromFile( mabFile )
{
  return getAddressBookNameFromFileName( mabFile.leafName );
} // getAddressBookNameFromFile



function getAddressBookNameFromFileName( mabFileName )
{
  if( mabFileName.lastIndexOf(".") >= 0 )
    mabName = mabFileName.substr( 0, mabFileName.lastIndexOf(".") );
  else
    mabName = mabFileName;
  mabName = unescape( mabName );

  return mabName;
} // getAddressBookNameFromFile



function importAddressBookViaFile( externMabFile, localMabFileName, onlynewer )
{
  var mabName;
  var mabFileName;
  var localMab;
  var deleteMab = false;

  if( !externMabFile.exists() ||
      externMabFile.leafName.toLowerCase().lastIndexOf( ".mab" ) < 0 )
    return false;

  if( localMabFileName == '' )
  {
    // find existing mab
    mabName = getAddressBookNameFromFile( externMabFile );
    localMab = getAddressBookByName( mabName );

    if( localMab )
    {
      // mab does exist => delete it before overwriting
      mabFileName = localMab.directoryProperties.fileName;
      deleteMab = true;
    }
    else
    {
      mabFileName = getNewAddressBookFileName( externMabFile.leafName );
    } // else
  }
  else
  {
    localMab = getAddressBook( localMabFileName );
    mabName = localMab.directoryProperties.description;
    mabFileName = localMabFileName;
    deleteMab = true;
  } // else

  if( deleteMab )
  {
    if( onlynewer )
    {
      var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
      localMabFile.appendRelativePath( mabFileName );

      if( localMabFile.lastModifiedTime >= externMabFile.lastModifiedTime )
        return true;
    } // if

    deleteAddressBook( mabFileName, true );
  } // if

  externMabFile.copyTo( profileDir, mabFileName );
  createAddressBook( mabName, mabFileName );
  
  return true;

} // importAddressBookViaFile



function importAllAddressBooksViaFile( externMabDir, onlynewer )
{
  var ret = true;
  
  if( externMabDir.exists() )
  {
    if( !externMabDir.isDirectory() )
      externMabDir = externMabDir.parent;

    var dirator = externMabDir.directoryEntries;
    
    while( dirator.hasMoreElements() )
    {
      var externMabFile = dirator.getNext().QueryInterface(Components.interfaces.nsILocalFile);

      if( !importAddressBookViaFile( externMabFile, "", onlynewer ) )
        ret = false;
    } // while

  } // if
  
  return ret;

} // importAllAddressBooksViaFile



function exportAddressBookViaFile( localMabFileName, externMabFile, onlynewer )
{
  var mabName;
  var localMab;
  
  if( localMabFileName == "" )
  {
    mabName = getAddressBookNameFromFile( externMabFile );
    localMab = getAddressBookByName( mabName );
    
    if( !localMab )
      return false;
      
    localMabFileName = localMab.directoryProperties.fileName;
  }
  else
  {
    localMab = getAddressBook( localMabFileName );
    mabName = localMab.directoryProperties.description;
  } // else

  var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
  localMabFile.appendRelativePath( localMabFileName );

  if( !localMabFile.exists() )
    return false;

  if( externMabFile.exists() &&
      externMabFile.isDirectory() )
  {
    var externMabFileName = escape( mabName ) + ".mab";
    externMabFile = externMabFile.clone().QueryInterface(Components.interfaces.nsILocalFile);
    externMabFile.appendRelativePath( externMabFileName );
  } // if

  if( externMabFile.exists() )
  {
    if( onlynewer )
    {
      if( localMabFile.lastModifiedTime <= externMabFile.lastModifiedTime )
        return true;
    } // if

    externMabFile.remove(false);
  } // if

  localMabFile.copyTo( externMabFile.parent, externMabFile.leafName );
  
  return true;

} // exportAddressBookViaFile



function exportAllAddressBooksViaFile( externMabDir, onlynewer )
{
  var ret = true;
  
  if( externMabDir.exists() )
  {
    if( !externMabDir.isDirectory() )
      externMabDir = externMabDir.parent;
      
    var cn = directory.childNodes;
    var aBook = cn.getNext();

    while( aBook != null )
    {
      var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);

      var localMabFileName = cur.directoryProperties.fileName;
      
      if( !exportAddressBookViaFile( localMabFileName, externMabDir, onlynewer ) )
        ret = false;

      try
      {
          aBook = cn.getNext();
      }
      catch(e)
      {
          break;
      } // catch
    } // while

  } // if
  
  return ret;

} // exportAllAddressBooksViaFile



function getNewAddressBookFileName( oldFileName )
{
  mabFileName = oldFileName;
  
  // dont overwrite other same named mabs
  var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
  localMabFile.appendRelativePath( mabFileName );
  var mabCount = 0;
  var oldMabFileName = mabFileName.substr( 0, mabFileName.lastIndexOf(".") );

  while( localMabFile.exists() )
  {
    mabCount++;
    mabFileName = oldMabFileName+mabCount+".mab";
    localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
    localMabFile.appendRelativePath( mabFileName );
  } // while

  return localMabFile.leafName;
  
} // getNewAddressBookFileName



