// License: http://creativecommons.org/licenses/by-nc-sa/2.0/

function SyncMabHandlerFTP( mabSync, parentSync )
{
  this._init( mabSync, parentSync );
}

////////////////////////////////////////////////////////////////////////////////

SyncMabHandlerFTP.prototype = 
{
    _mabSync:null,
    _ioService:null,
    _syncService:null,
    _downloadHostDirectoryFunc:null,
    _mabSyncHandler:null,
    _parentSync:null,
    _hostDirectory:null,
  

////////////////////////////////////////////////////////////////////////////////
// Initialize

  _init:function( mabSync, parentSync )
  {
    this._mabSync = mabSync;
    this._ioService = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
    this._parentSync = parentSync;
  },

////////////////////////////////////////////////////////////////////////////////

  _transmissionDone:function( id )
  {
    if( this._mabSyncHandler != null )
    {
      this._mabSyncHandler[id] = null;

      for( i = 0; i < this._mabSyncHandler.length; i++ )
      {
        if( this._mabSyncHandler[i] != undefined &&
            this._mabSyncHandler[i] != null )
          return;
      } // for
    } // if
    
    if( this._parentSync )
      this._parentSync._transmissionDone( this._mabSync.syncId );
    else
      mabSyncHandler[this._mabSync.syncId] = null;

  },

////////////////////////////////////////////////////////////////////////////////

  _getHostDirectory:function()
  {
    if( this._hostDirectory )
      return this._hostDirectory;
      
    if( this._parentSync )
      return this._parentSync._getHostDirectory();
      
    return null;
  },

////////////////////////////////////////////////////////////////////////////////
// Overwrite the FTP server data file.

  push:function() 
  {
    var ret = true;
    
    if( this._mabSync.filePath == "" )
      return false;
      
    if( this._mabSync.mabToExport == "" )
    {
      if( this._mabSync.exportNew &&
          this._getHostDirectory() == null )
        this._downloadHostDirectory( "_pushAllMab" );
      else
        ret = this._pushAllMab();
    }
    else
    {
      if( this._mabSync.exportNew &&
          this._getHostDirectory() == null )
        this._downloadHostDirectory( "_pushMab" );
      else
        ret = this._pushMab();
    } // else

    return ret;
  },

////////////////////////////////////////////////////////////////////////////////

  _pushMab:function()
  {
    var ret = true;

    var mabFileName = this._mabSync.mabToExport;
    var localMab;
    var mabName;
    var externMabFileName;

    if( this._mabSync.filePath.lastIndexOf("/") == this._mabSync.filePath.length-1 )
    {
      localMab = getAddressBook( mabFileName );
      mabName = localMab.directoryProperties.description;
      externMabFileName = escape( mabName ) + ".mab";
      this._mabSync.filePath += externMabFileName;
    }
    else
    {
      externMabFileName = this._mabSync.filePath.substr( this._mabSync.filePath.lastIndexOf("/")+1 );
    } // else

    var mabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
    mabFile.appendRelativePath( mabFileName );
    if( !mabFile.exists() )
        return false;
        
    if( this._mabSync.exportNew )
    {
      var externFileData;
      var hostdir = this._getHostDirectory();
      if( hostdir )
      {
        externFileData = hostdir.getLeafByFileName( externMabFileName );
        if( externFileData )
          if( mabFile.lastModifiedTime <= externFileData.lastmodified )
          {
            this._transmissionDone( this._mabSync.syncId );
            return true;
          } // if
      }
      else
        return false;
    } // if

    var mabFileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].
        createInstance(Components.interfaces.nsIScriptableInputStream);

    mabFileStream.init( mabFile, 1, 0, false );
    sstream.init(mabFileStream);

    var source = "";
    var str = sstream.read(4096);
    while( str.length > 0 )
    {
      source += str;
      str = sstream.read(4096);
    } // while

    sstream.close();
    mabFileStream.close();

    this._syncService = new SyncMabSynchronizerUploadService();
    return this._syncService.start(
        source,
        this._uri,
        "text/xml",
        this
        );

    return ret;
  },

////////////////////////////////////////////////////////////////////////////////

  _pushAllMab:function()
  {
    var ret = true;

    // export all abooks
    var mabFileName = this._mabSync.mabToExport;
    var localMab;
    var mabName;
    var cn = directory.childNodes;
    var aBook = cn.getNext();
    var syncCount = 0;
    var externDir = this._mabSync.filePath.substr( 0, this._mabSync.filePath.lastIndexOf("/")+1 );
    this._mabSyncHandler = new Array();
    var errors = 0;

    while( aBook != null )
    {
      var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);
      mabFileName = cur.directoryProperties.fileName;
      localMab = getAddressBook( mabFileName );
      mabName = localMab.directoryProperties.description;

      var mabSync = new MabSyncData( this._mabSync );
      mabSync.syncId = syncCount;
      mabSync.mabToExport = mabFileName;
      mabSync.filePath = externDir + escape(mabName) + ".mab";
      this._mabSyncHandler[syncCount] = new SyncMabHandlerFTP( mabSync, this );
      if( this._mabSyncHandler[syncCount].push() == false )
        error++;

      syncCount++;

      try
      {
          aBook = cn.getNext();
      }
      catch(e)
      {
          break;
      } // catch
    } // while

    if( errors == syncCount )
      ret = false;

    return ret;
  },

////////////////////////////////////////////////////////////////////////////////
// Overwrite Mab with the FTP server data file.

  pull:function() 
  {
    var ret = true;
    
    if( this._mabSync.filePath.lastIndexOf("/") == this._mabSync.filePath.length-1 )
    {
      if( this._getHostDirectory() == null )
        this._downloadHostDirectory( "_pullAllMab" );
    }
    else
    {
      if( this._mabSync.importNew &&
          this._getHostDirectory() == null )
        this._downloadHostDirectory( "_pullMab" );
      else
        ret = this._pullMab();
    } // else
    
    return ret;

  },

////////////////////////////////////////////////////////////////////////////////

  _pullMab:function()
  {
    if( this._mabSync.importNew )
    {
      var externMabFileName = this._mabSync.filePath.substr( this._mabSync.filePath.lastIndexOf("/")+1 );
      var externFileData;
      var hostdir = this._getHostDirectory();
      var mabName = getAddressBookNameFromFileName(externMabFileName);
      var localMab = getAddressBookByName( mabName );
      
      if( hostdir )
      {
        if( localMab )
        {
          externFileData = hostdir.getLeafByFileName( externMabFileName );
          
          if( externFileData )
          {
            var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
            localMabFile.appendRelativePath( localMab.directoryProperties.fileName );
            
            if( localMabFile.lastModifiedTime >= externFileData.lastmodified )
            {
              this._transmissionDone( this._mabSync.syncId );
              return true;
            } // if
          } // if
        } // if
      }
      else
        return false;
    } // if

    this._syncService = new SyncMabSynchronizerDownloadService();
    return this._syncService.start(
        this._uri,
        this
        );
  },

////////////////////////////////////////////////////////////////////////////////

  _pullAllMab:function()
  {
    var ret = true;
    
    // import all abooks
    var mabFileName = this._mabSync.mabToImport;
    var mabName;
    var externDir = this._mabSync.filePath.substr( 0, this._mabSync.filePath.lastIndexOf("/")+1 );
    var errors = 0;
    var hostdir = this._getHostDirectory();
    var syncCount = 0;
    this._mabSyncHandler = new Array();

    for( filei = 0; filei < hostdir.leaves.length; filei++ )
    {
      mabFileName = hostdir.leaves[filei].filename;

      if( mabFileName.lastIndexOf(".mab") > 0 )
      {
        var mabSync = new MabSyncData( this._mabSync );
        mabSync.syncId = syncCount;
        mabSync.filePath = externDir + mabFileName;
        this._mabSyncHandler[syncCount] = new SyncMabHandlerFTP( mabSync, this );
        if( this._mabSyncHandler[syncCount].pull() == false )
          error++;

        syncCount++;
      } // if
      
    } // while

    if( errors == syncCount )
      ret = false;

    return ret;
  },

////////////////////////////////////////////////////////////////////////////////
// download directory

  _downloadHostDirectory:function( functogo )
  {
    var oldPath = this._mabSync.filePath;
    this._mabSync.filePath = this._mabSync.filePath.substr( 0, this._mabSync.filePath.lastIndexOf("/")+1 );

    if( this._mabSync.filePath == "" )
      return false;

    this._downloadHostDirectoryFunc = functogo;

    this._syncService = new SyncMabSynchronizerDownloadService();
    this._syncService.start(
        this._uri,
        this
        );
        
    this._mabSync.filePath = oldPath;
  },

////////////////////////////////////////////////////////////////////////////////
// Gets the URI for transferring the data.

  get _uri() 
  {
    var uri = this._mabSync.syncLocation + "://";

    if (this._mabSync.syncUsername != "") {
      uri += this._mabSync.syncUsername;
      if (this._mabSync.syncPassword != "") {
        uri += ":" + this._mabSync.syncPassword;
      }
      uri += "@";
    }

    var ftpPath = this._mabSync.filePath;
    if (! ftpPath.charAt(0) == '/') {
      ftpPath = "/" + ftpPath;
    }
    
    if( ftpPath.charAt(ftpPath.length-1) != '/')
    {
      var slpos = ftpPath.lastIndexOf( '/' ) + 1;
      var dir = ftpPath.substr( 0, slpos );
      var leaf = ftpPath.substr( slpos );
      ftpPath = dir + escape( leaf );
    } // if

    uri += this._mabSync.syncHost + ftpPath;
    return this._ioService.newURI(uri, "UTF-8", null);
  },

////////////////////////////////////////////////////////////////////////////////

  _uploadCallback:function(stat, err) 
  {
    if( stat == "done" )
    {
      this._transmissionDone( this._mabSync.syncId );
    } // if

  },

////////////////////////////////////////////////////////////////////////////////

  _downloadCallback:function(stat, err) 
  {
    if( stat == "done" )
    {
        if( err == 0 )
        {
          if( this._downloadHostDirectoryFunc )
          {
            this._hostDirectory = new HostDirectory();
            this._hostDirectory.init( this._syncService.data );
            eval( "this." + this._downloadHostDirectoryFunc + "();" );
            this._downloadHostDirectoryFunc = null;
          }
          else
          {
            var mabFileName;
            var mabName;
            var deleteMab = false;
            var localMabFile;

            if( this._mabSync.mabToImport == "" )
            {
                mabFileName = this._mabSync.filePath.substr( this._mabSync.filePath.lastIndexOf("/")+1 );
                mabName = getAddressBookNameFromFileName( mabFileName );

                localMab = getAddressBookByName( mabName );
                
                if( localMab )
                {
                  mabFileName = localMab.directoryProperties.fileName;
                  deleteMab = true;
                }
                else
                {
                  mabFileName = getNewAddressBookFileName( mabFileName );
                } // else
            }
            else
            {
                mabFileName = this._mabSync.mabToImport;
                mabName = getAddressBook( mabFileName ).directoryProperties.description;
                deleteMab = true;
            } // else

            if( deleteMab )
            {
                deleteAddressBook( mabFileName, true );
            } // if

            localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
            localMabFile.appendRelativePath( mabFileName );

            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(localMabFile, 0x02 | 0x08 | 0x20, 0664, 0);
            foStream.write(this._syncService.data, this._syncService.length);
            foStream.close();

            createAddressBook( mabName, mabFileName );
            
            this._transmissionDone( this._mabSync.syncId );
          } // else
        } // if
    } // if
   
  },

////////////////////////////////////////////////////////////////////////////////
};


function HostDirectory()
{
  this.path = '';
  this.leaves = new Array();
  
} // HostDirectory


HostDirectory.prototype.init = function( data )
{
  var dirlines = data.split( "\n" );
  var fieldnames;
  var field_filename;
  var field_contentlength;
  var field_lastmodified;
  var field_filetype;

  for( i = 0; i < dirlines.length; i++ )
  {
    var line = dirlines[i].split( " " );

    if( line[0] == '300:' )
    {
      // path
      this.path = line[1];
    }
    else
    if( line[0] == '200:' )
    {
      // fieldnames
      for( j = 1; j < line.length; j++ )
      {
        if( line[j] == 'filename' )
          field_filename = j;
        else
        if( line[j] == 'content-length' )
          field_contentlength = j;
        else
        if( line[j] == 'last-modified' )
          field_lastmodified = j;
        else
        if( line[j] == 'file-type' )
          field_filetype = j;
      } // for
    }
    else
    if( line[0] == '201:' )
    {
      // file data
      var leafid = this.leaves.length;
      this.leaves[leafid] = new HostDirectoryLeaf();
      this.leaves[leafid].setFileName( line[field_filename] );
      this.leaves[leafid].contentlength = line[field_contentlength];
      this.leaves[leafid].setLastModified( line[field_lastmodified] );
      this.leaves[leafid].filetype = line[field_filetype];
    }
  } // for

} // HostDirectory.prototype.init



HostDirectory.prototype.getLeafByFileName = function( fileName )
{
  for( i = 0; i < this.leaves.length; i++ )
  {
    if( this.leaves[i].filename == fileName )
      return this.leaves[i];
  } // for
  
  return null;
  
} // HostDirectory.prototype.getLeafByFileName



function HostDirectoryLeaf()
{
  this.filename = '';
  this.contentlength = 0;
  this.lastmodified = 0;
  this.filetype = '';
} // HostDirectoryLeaf


HostDirectoryLeaf.prototype.isDirectory = function()
{
  return (this.field_filetype == 'DIRECTORY');
} // HostDirectoryLeaf.prototype.isDirectory


HostDirectoryLeaf.prototype.isFile = function()
{
  return (this.field_filetype == 'FILE');
} // HostDirectoryLeaf.prototype.isFile


HostDirectoryLeaf.prototype.setFileName = function( filename )
{
  if( filename.charAt(0) == '"' )
    filename = filename.substr( 1 );
  if( filename.charAt(filename.length-1) == '"' )
    filename = filename.substr( 0, filename.length-1 );
  this.filename = unescape( filename );
} // HostDirectoryLeaf.prototype.setFileName


HostDirectoryLeaf.prototype.setLastModified = function( lastmodified )
{
  var date = new Date( unescape(lastmodified) );
  this.lastmodified = date.getTime();
} // HostDirectoryLeaf.prototype.setLastModified


